# -*- coding: utf-8 -*-

import os, sys
import openslide


__all__ = ["create_pyramidal_img",
            "load_wsi_head",
            ]


def create_pyramidal_img(img_path, save_dir):
    """ Convert normal image to pyramidal image.

    Parameters
    -------
    img_path: str
        Whole slide image path (absolute path is needed)
    save_dir: str
        Location of the saved the generated pyramidal image with extension tiff,
        (absolute path is needed)

    Returns
    -------
    status: int
        The status of the pyramidal image generation (0 stands for success)

    Notes
    -------
    ImageMagick need to be preinstalled to use this function.
    >>> sudo apt-get install imagemagick

    Examples
    --------
    >>> img_path = os.path.join(PRJ_PATH, "test/data/Images/CropBreastSlide.tif")
    >>> save_dir = os.path.join(PRJ_PATH, "test/data/Slides")
    >>> status = pyramid.create_pyramidal_img(img_path, save_dir)
    >>> assert status == 0

    """

    convert_cmd = "convert " + img_path
    convert_option = " -compress jpeg -quality 90 -define tiff:tile-geometry=256x256 ptif:"
    img_name = os.path.basename(img_path)
    convert_dst = os.path.join(save_dir, os.path.splitext(img_name)[0] + ".tiff")
    status = os.system(convert_cmd + convert_option + convert_dst)

    return status


def load_wsi_head(wsi_img_path):
    """ Load the header meta data of whole slide pyramidal image.

    Parameters
    -------
    wsi_img_path: str
        The path to whole slide image

    Returns
    -------
    wsi_header: slide class
        Meta information of whole slide image

    """

    wsi_header = openslide.OpenSlide(wsi_img_path)

    return wsi_header



# -*- coding: utf-8 -*-

import os, sys
import numpy as np
import itertools

__all__ = ['wsi_patch_splitting',]


def wsi_patch_splitting(wsi_h, wsi_w, length, overlay=True):
    '''Spltting whole slide image to be patch-wise.
    '''
    coors_arr = []

    # splitting in both w and h direction with overlay
    def split_patch_overlay(ttl_len, sub_len):
        p_sets = []
        if ttl_len < sub_len:
            return p_sets
        if ttl_len == sub_len:
            p_sets.append(0)
            return p_sets

        p_num = int(np.ceil(ttl_len * 1.0 / sub_len))
        overlap_len = (p_num * sub_len - ttl_len) * 1.0 / (p_num - 1)
        extend_len = sub_len - overlap_len
        for ind in np.arange(p_num):
            p_sets.append(int(round(extend_len * ind)))
        return p_sets

    # splitting in both w and h direction with overlay
    def split_patch_no_overlay(ttl_len, sub_len):
        p_sets = []
        if ttl_len < sub_len:
            return p_sets
        if ttl_len == sub_len:
            p_sets.append(0)
            return p_sets

        p_num = int(np.floor(ttl_len * 1.0 / sub_len))
        p_sets = [ele*sub_len for ele in np.arange(p_num)]
        return p_sets

    if overlay == True:
        h_sets = split_patch_overlay(wsi_h, length)
        w_sets = split_patch_overlay(wsi_w, length)
    else:
        h_sets = split_patch_no_overlay(wsi_h, length)
        w_sets = split_patch_no_overlay(wsi_w, length)

    # combine points in both w and h direction
    if len(w_sets) > 0 and len(h_sets) > 0:
        coors_arr = list(itertools.product(w_sets, h_sets))
    else:
        raise Exception("Cannot splitting")


    return coors_arr



# -*- coding: utf-8 -*-
import os, sys, pdb
from setuptools import setup, find_packages

import pyslide

PKG_NAME = "pyslide"
VERSION = pyslide.__version__
DESCRIPTION = "Whole slide image analysis toolbox."
HOMEPAGE = "https://github.com/PingjunChen/pyslide"
LICENSE = "MIT"
AUTHOR_NAME = "Pingjun Chen"
AUTHOR_EMAIL = "chenpingjun@gmx.com"

REQS = ""
with open('requirements.txt') as f:
    REQS = f.read().splitlines()

CLASSIFIERS = [
    'Development Status :: 1 - Planning',
    'Intended Audience :: Developers',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: Apache Software License',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Topic :: Scientific/Engineering',
]

args = dict(
    name=PKG_NAME,
    version=VERSION,
    description=DESCRIPTION,
    url=HOMEPAGE,
    license=LICENSE,
    author=AUTHOR_NAME,
    author_email=AUTHOR_EMAIL,
    packages=find_packages(),
    install_requires=REQS,
    classifiers= CLASSIFIERS,
)

setup(**args)
